-- cron：0 0 */1 * * ?
-- 任务名称：EVENT_BIG_CATEGORY_STATISTIC
-- 任务描述：1小时粒度事件统计
-- 输出数据表：t_analysis_event
-- 依赖包：flink-connector-jdbc_2.11-1.14.3.jar、mysql-connector-java-8.0.21.jar


set table.exec.sink.not-null-enforcer=drop;

CREATE TABLE t_event_info(
                             oid BIGINT  NOT NULL,
                             type STRING,
                             confidence INT,
                             lng DOUBLE,
                             lat DOUBLE,
                             start_time TIMESTAMP(3),
                             end_time TIMESTAMP(3),
                             detect_time TIMESTAMP(3),
                             plate_no STRING,
                             grade INT,
                             object_type STRING,
                             category STRING,
                             source STRING,
                             event_serial_number STRING,
                             data_status STRING,
                             global_id STRING,
                             station_id INT,
                             lane_id STRING,
                             rid STRING,
                             segment_id STRING,
                             cross_id STRING,
                             ruksj TIMESTAMP(3),
                             PRIMARY KEY(oid) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8',
      'table-name' = 't_event_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

CREATE TABLE t_analysis_event (
    -- oid BIGINT  NOT NULL,
                                  event_category STRING,
                                  event_type STRING,
                                  event_number INT,
                                  statistic_time TIMESTAMP(3),
                                  window_start_time TIMESTAMP(3),
                                  window_end_time TIMESTAMP(3),
                                  granularity STRING,
                                  ratio DECIMAL(6,2),
                                  object_type STRING,
                                  cross_id STRING,
                                  rid STRING,
                                  segment_id STRING,
                                  lane_id STRING,
                                  PRIMARY KEY(cross_id,rid,segment_id,lane_id,granularity,window_start_time,event_type) NOT ENFORCED
)  WITH (
       'connector' = 'jdbc',
       'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8',
       'table-name' = 't_analysis_event',
       'username' = 'root',
       'password' = 'Wanji300552'
       );

-- 一小时粒度统计
insert into t_analysis_event(event_category,event_type,event_number,window_start_time,window_end_time,granularity,object_type,cross_id,rid,segment_id,lane_id)
SELECT  category,type,CAST(count(*) AS INT) e_count,
        TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:00:00')) window_start_time,
        TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:00:00')),
        -- TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:00:00')) window_end_time,
        '1h' granularity,object_type,cross_id,rid,segment_id,lane_id
FROM  t_event_info
      -- WHERE start_time >= TO_TIMESTAMP(date_format(TIMESTAMPADD(MINUTE,-5,LOCALTIMESTAMP),'yyyy-MM-dd HH:00:00')) and start_time < TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:00:00'))
WHERE ruksj>=TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:00:00')) and ruksj<TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:00:00'))
GROUP BY category,object_type,type,cross_id,rid,segment_id,lane_id
;

create table t_base_lane_info (
                                  cross_id VARCHAR, -- 路口ID
                                  id VARCHAR ,-- 车道ID
                                  rid VARCHAR, -- 路段ID
                                  dir INT,-- 路段方向
                                  sort INT, -- 车道编号
                                  turn INT, -- 车道功能
                                  segment_id VARCHAR, -- 渠化ID
                                  type INT, -- 1：路段车道 2：进口车道 3：出口车道
                                  category INT
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_cs?characterEncoding=UTF-8',
      'table-name' = 't_base_lane_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

CREATE TABLE t_analysis_cross_non_motor (
                                            id STRING,
                                            cross_id STRING,
                                            turn_no INT,
                                            type INT,
                                            number INT,
                                            granularity STRING,
                                            window_start_time TIMESTAMP(3),
                                            window_end_time TIMESTAMP(3),
                                            create_date TIMESTAMP(3),
                                            PRIMARY KEY(cross_id,turn_no,type,granularity,window_start_time) NOT ENFORCED
)  WITH (
       'connector' = 'jdbc',
       'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
       'table-name' = 't_analysis_cross_non_motor',
       'username' = 'root',
       'password' = 'Wanji300552'
       );



-- 非机动车效率：非机动车道 闯红灯数据
INSERT INTO t_analysis_cross_non_motor(cross_id,turn_no,type,number,granularity,window_start_time,window_end_time,create_date)
SELECT  a.cross_id,
        b.turn,
        3 as type,
        CAST(count(*) AS INT) e_count,
        '1h' granularity,
        TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:mm:00')) window_start_time,
        TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:mm:00')) window_end_time,
        LOCALTIMESTAMP
FROM  t_event_info a left join t_base_lane_info b on a.lane_id=b.id
      -- WHERE start_time >= TO_TIMESTAMP(date_format(TIMESTAMPADD(MINUTE,-5,LOCALTIMESTAMP),'yyyy-MM-dd HH:00:00')) and start_time < TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:00:00'))
WHERE a.category='2'  and ruksj>=TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:mm:00')) and ruksj<TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:mm:00'))
GROUP BY a.cross_id,b.turn
;


-- 行人闯红灯数量、未礼让行人机动车数量
CREATE TABLE t_analysis_cross_pedestrian (
                                             cross_id STRING,
                                             avg_pass_rate DECIMAL(6,2),
                                             avg_pass_time DECIMAL(6,2),
                                             avg_wait_time DECIMAL(6,2),
                                             avg_wait_people_number INT,
                                             run_red_number INT,
                                             wlrxr_non_motor_number INT,
                                             wlrxr_motor_number INT,
                                             accelerate_pass_rate DECIMAL(6,2),
                                             granularity STRING,
                                             window_start_time TIMESTAMP(3),
                                             window_end_time TIMESTAMP(3),
                                             create_date TIMESTAMP(3),
                                             PRIMARY KEY(cross_id,granularity,window_start_time) NOT ENFORCED
)  WITH (
       'connector' = 'jdbc',
       'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
       'table-name' = 't_analysis_cross_pedestrian',
       'username' = 'root',
       'password' = 'Wanji300552'
       );

INSERT INTO t_analysis_cross_pedestrian(cross_id,run_red_number,wlrxr_motor_number,granularity,window_start_time,window_end_time,create_date)
SELECT t1.cross_id,t1.run_red_number,t2.wlrxr_motor_number,
       '15m' granularity,
       TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:mm:00')) window_start_time,
       TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:mm:00')) window_end_time,
       LOCALTIMESTAMP
FROM
    (
        SELECT  a.cross_id, CAST(count(*) AS INT),0)  run_red_number
    FROM  t_event_info a
WHERE a.type='14' -- and ruksj>=TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:mm:00')) and ruksj<TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:mm:00'))
GROUP BY a.cross_id
    ) t1
    FULL JOIN
    (
SELECT  a.cross_id, CAST(count(*) AS INT),0)  wlrxr_motor_number
FROM  t_event_info a
WHERE a.type='31' -- and ruksj>=TO_TIMESTAMP(date_format(TIMESTAMPADD(HOUR,-1,LOCALTIMESTAMP),'yyyy-MM-dd HH:mm:00')) and ruksj<TO_TIMESTAMP(date_format(LOCALTIMESTAMP,'yyyy-MM-dd HH:mm:00'))
GROUP BY a.cross_id
    ) t2
ON t1.cross_id=t2.cross_id
;