检查点参数设置：
-checkpointInterval 120000 -checkpointDir file:///data/soft/flink/checkpoints

CREATE TABLE kafka_vehicle_track_source (
                                            `timeStamp` VARCHAR,
                                            globalTimeStamp BIGINT,
                                            participantNum INT,
                                            orgCode VARCHAR,
                                            e1FrameParticipant ARRAY<
                                                ROW<id INT,licenseColor INT,picLicense VARCHAR,originalColor INT,originalType INT,longitude DOUBLE,latitude DOUBLE,speed DOUBLE,altitude DOUBLE,courseAngle DOUBLE,baseLocation INT,waitingTime INT,stopNum INT,roadnet MAP<String,String>>
                                                >,
                                            ts_ltz AS TO_TIMESTAMP_LTZ(globalTimeStamp, 3),
                                            WATERMARK FOR ts_ltz AS ts_ltz - INTERVAL '0' SECOND,
                                            process_time AS PROCTIME ()
) WITH (
      'connector' = 'kafka', -- 使用 kafka connector
      'topic' = 'JNMatchResultMiniData',  -- kafka topic
      'properties.bootstrap.servers' = '10.102.1.182:9092',  -- broker连接信息
      'properties.group.id' = 'frame-data-source_to_flie', -- 消费kafka的group_id
      'properties.enable.auto.commit' = 'true',
      'properties.auto.commit.interval.ms' = '1000',
      -- 'scan.startup.mode' = 'group-offsets',
      -- 'scan.startup.mode' = 'latest-offset',
      'scan.startup.mode' = 'earliest-offset',
      -- 'scan.startup.timestamp-millis' = '1689436800000',
      -- 'scan.startup.mode' = 'timestamp',  -- 读取数据的位置
      'format' = 'json',  -- 数据源格式为 json
      'json.fail-on-missing-field' = 'false', -- 字段丢失任务不失败
      'json.ignore-parse-errors' = 'true'  -- 解析失败跳过
      ) ;

CREATE TABLE kafka_vehicle_track_sink (
                                          frameTime VARCHAR,
                                          globalTimeStamp BIGINT,
                                          participantNum INT,
                                          orgCode VARCHAR,
                                          e1FrameParticipant ARRAY<
                                              ROW<id INT,licenseColor INT,picLicense VARCHAR,originalColor INT,originalType INT,longitude DOUBLE,latitude DOUBLE,speed DOUBLE,altitude DOUBLE,courseAngle DOUBLE,baseLocation INT,waitingTime INT,stopNum INT,roadnet MAP<String,String>>
                                              >,
                                          dt VARCHAR,
                                          ht VARCHAR,
                                          mt VARCHAR
) PARTITIONED BY (dt,ht,mt)
WITH (
    'connector' = 'filesystem', -- 保存到文件系统
    'path' = 'file:///data/track_partitions/',
    -- 'name' = 'frame_track_$dt$mt',
    'format' = 'json',  -- 数据源格式为 json
    'sink.partition-commit.policy.kind'='success-file',
    -- 'partition.time-extractor.timestamp-formatter' = 'yyyy-MM-dd_HHmmss',
    'partition.time-extractor.timestamp-pattern'='$dt $ht:$mt:00',
    'sink.partition-commit.trigger'='partition-time',
    'sink.partition-commit.delay'='1s' ,
    'sink.rolling-policy.rollover-interval'='30s',
    'sink.rolling-policy.check-interval'='10s',
    'sink.rolling-policy.file-size' = '1024MB',
    'sink.partition-commit.watermark-time-zone'='Asia/Shanghai',
    'auto-compaction'='true'
    -- ,'sink.partition-commit.success-file.name'='frame_track_$dt:$mt'
    -- ,'partition.default-name'='frame_track_$dt$mt'
) ;
insert into kafka_vehicle_track_sink(frameTime,globalTimeStamp,participantNum,orgCode,e1FrameParticipant,dt,ht,mt)
select
    `timeStamp`,
    globalTimeStamp,
    participantNum,
    orgCode,
    e1FrameParticipant,
    DATE_FORMAT(ts_ltz, 'yyyy-MM-dd'),
    LPAD(CAST(HOUR(ts_ltz) AS VARCHAR),2,'0'),
    LPAD(CAST(MINUTE(ts_ltz)/10*10 AS VARCHAR),2,'0')
from kafka_vehicle_track_source
;

