set table.exec.sink.not-null-enforcer=drop;
-- 根据wkt坐标计算路段长度，wkt格式x1,y1;x2,y2;...
CREATE FUNCTION lineLengthUdf AS 'com.wanji.indicators.udf.LineLengthUdf';

-- 数据源kafka数据
CREATE TABLE source_gloabl_lane_cycle_data (
                                               `timeStamp` VARCHAR,
                                               orgCode VARCHAR,
                                               laneNum INT,
                                               eventList ARRAY<ROW<laneId VARCHAR,
                                               traffic_flow  INT,
                                               trolley_equivalent  INT,
                                               lane_flow_rate  DOUBLE,
                                               mean_v DOUBLE,
                                               vehicle_nums_ratio_mean  DOUBLE,
                                               vehicle_length_ratio_mean  DOUBLE,
                                               time_occupancy DOUBLE,
                                               static_queue_length_max DOUBLE,
                                               static_queue_length_min DOUBLE,
                                               dynamic_queue_length_max DOUBLE,
                                               dynamic_queue_length_min DOUBLE,
                                               light_green_start_queue_length DOUBLE,
                                               light_green_finish_queue_length DOUBLE,
                                               green_light_efficiency DOUBLE,
                                               lane_saturation_flow_rate INT,
                                               lane_capacity INT,
                                               lane_saturation DOUBLE,
                                               lane_no_stop_rate DOUBLE,
                                               lane_one_stop_rate DOUBLE,
                                               lane_two_stop_rate DOUBLE,
                                               lane_three_stop_rate DOUBLE,
                                               mean_delay DOUBLE,
                                               mean_stops_number DOUBLE,
                                               traffic_flow_A INT,
                                               traffic_flow_B INT,
                                               traffic_flow_C INT,
                                               mean_len DOUBLE,
                                               time_headway_section_mean DOUBLE,
                                               traffic_infomation INT
                                                   >>,
                                               ts_ltz AS TO_TIMESTAMP_LTZ(UNIX_TIMESTAMP(`timeStamp`,'yyyy-MM-dd HH:mm:ss:SSS'), 0),
                                               WATERMARK FOR ts_ltz AS ts_ltz - INTERVAL '5' SECOND,
                                               process_time AS PROCTIME ()
) WITH (
      'connector' = 'kafka', -- 使用 kafka connector
      'topic-pattern' = 'JN(.*)LanePeriodicData',  -- kafka topic
      'properties.bootstrap.servers' = '192.168.2.103:9092',  -- broker连接信息
      'properties.group.id' = 't_global_lane_cycle_data_10m', -- 消费kafka的group_id
      'properties.enable.auto.commit' = 'true',
      'properties.auto.commit.interval.ms' = '1000',
      'scan.startup.mode' = 'group-offsets',
      -- 'scan.startup.mode' = 'latest-offset',  -- 读取数据的位置
      --  'scan.startup.mode' = 'earliest-offset',
      -- 'scan.startup.mode' = 'group-offsets',
      --  'scan.startup.mode' = 'timestamp',  -- 读取数据的位置
      'scan.startup.timestamp-millis' = '1683648000000', -- 设定消费的时间戳起点
      'format' = 'json',  -- 数据源格式为 json
      'json.fail-on-missing-field' = 'false', -- 字段丢失任务不失败
      'json.ignore-parse-errors' = 'true'  -- 解析失败跳过
      ) ;
-- Sink表
create table t_analysis_rid_lane_indicators
(
    cross_id VARCHAR,
    cross_name VARCHAR,
    rid VARCHAR,
    rid_dir INT,
    segment_id  VARCHAR,
    lane_id VARCHAR,
    lane_no INT,
    turn_dir_no INT,
    flow INT,
    saturation DECIMAL(6,2),
    delay_time DECIMAL(6,2),
    speed DECIMAL(6,2),
    queue_length INT,
    stop_num DECIMAL(6,2),
    once_stop_rate  DECIMAL(6,2),
    second_stop_rate DECIMAL(6,2),
    traffic_state VARCHAR,
    statistic_time TIMESTAMP(3),
    granularity VARCHAR,
    ruksj TIMESTAMP(3),
    trolley_equivalent   DECIMAL(6,2),
    lane_flow_rate  DECIMAL(6,2),
    vehicle_nums_ratio_mean  DECIMAL(6,2),
    vehicle_length_ratio_mean  DECIMAL(6,2),
    time_occupancy  DECIMAL(6,2),
    static_queue_length_max  DECIMAL(6,2),
    static_queue_length_min  DECIMAL(6,2),
    dynamic_queue_length_max  DECIMAL(6,2),
    dynamic_queue_length_min  DECIMAL(6,2),
    light_green_start_queue_length  DECIMAL(6,2),
    light_green_finish_queue_length  DECIMAL(6,2),
    green_light_efficiency  DECIMAL(6,2),
    saturation_flow_rate  INT,
    capacity  INT,
    no_stop_rate  DECIMAL(6,2),
    three_stop_rate  DECIMAL(6,2),
    traffic_flow_A INT,
    traffic_flow_B INT,
    traffic_flow_C INT,
    mean_len DECIMAL(6,2),
    time_headway_section_mean DECIMAL(6,2),
    traffic_infomation INT,
    window_start_time TIMESTAMP(3),
    window_end_time TIMESTAMP(3),
    dt INT,
    PRIMARY KEY (rid,lane_id,granularity,window_end_time,dt) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?rewriteBatchedStatements=true&characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_rid_lane_indicators',
      'username' = 'root',
      'password' = 'Wanji300552',
      'sink.buffer-flush.max-rows' = '0',
      'sink.buffer-flush.interval' = '1000'
      );
create table t_base_lane_info (
                                  cross_id VARCHAR, -- 路口ID
                                  id VARCHAR ,-- 车道ID
                                  rid VARCHAR, -- 路段ID
                                  dir INT,-- 路段方向
                                  sort INT, -- 车道编号
                                  turn INT, -- 车道功能
                                  segment_id VARCHAR, -- 渠化ID
                                  type INT -- 1：路段车道 2：进口车道 3：出口车道
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8',
      'table-name' = 't_base_lane_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );
-- 路段rid表
create table t_base_rid_info (
                                 id VARCHAR ,-- 路段ID
                                 in_dir INT, -- 路段方向
                                 end_cross_id VARCHAR, -- 终点路口
                                 length DOUBLE, -- 路段长度
                                 wkt VARCHAR
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8',
      'table-name' = 't_base_rid_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );
-- 路口表
create table t_base_cross_info (
                                   id VARCHAR ,-- 路口ID
                                   `name` VARCHAR, -- 路口名称
                                   is_signal INT --是否为信号灯路口
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8',
      'table-name' = 't_base_cross_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

-- 自由流速度
CREATE TABLE t_cross_rid_freespeed (
                                       cross_id VARCHAR,
                                       rid VARCHAR,
                                       free_speed DECIMAL(6,2),
                                       window_start_time TIMESTAMP(3),
                                       window_end_time TIMESTAMP(3)
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_cross_rid_freespeed',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

-- 延误判定拥堵配置
CREATE TABLE t_config_congest_grade (
                                        min_delay_time DECIMAL(6,2),
                                        max_delay_time DECIMAL(6,2),
                                        traffic_state VARCHAR,
                                        is_signal INT
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_config_congest_grade',
      'username' = 'root',
      'password' = 'Wanji300552'
      );



-- 创建视图
CREATE VIEW v_kafka_cycle_data AS
SELECT b.cross_id,
       d.`name` as cross_name,
       b.rid,
       c.in_dir,
       b.segment_id,
       t.laneId,
       b.sort as lane_no,
       b.turn,
       t.traffic_flow,
       t.lane_saturation,
       t.mean_delay,
       t.mean_v,
       CAST(ROUND(t.static_queue_length_max,0) AS INT) queue_length,
       t.mean_stops_number,
       t.lane_one_stop_rate,
       t.lane_two_stop_rate,
       --  (select traffic_state from t_config_congest_grade tg where tg.is_signal=d.is_signal and t.mean_delay>=tg.min_delay_time and t.mean_delay<tg.max_delay_time) as traffic_state,
       (case
            when d.is_signal=1 and t.mean_delay>=0 and t.mean_delay<55 then '1'
            when d.is_signal=1 and t.mean_delay>=55 and t.mean_delay<100 then '2'
            when d.is_signal=1 and t.mean_delay>=100 and t.mean_delay<145 then '3'
            when d.is_signal=1 and t.mean_delay>=145  then '4'
            when d.is_signal=0 and t.mean_delay>=0 and t.mean_delay<35 then '1'
            when d.is_signal=0 and t.mean_delay>=35 and t.mean_delay<50 then '2'
            when d.is_signal=0 and t.mean_delay>=50 and t.mean_delay<70 then '3'
            when d.is_signal=0 and t.mean_delay>=70  then '4'
           end
           ) traffic_state,
       -- '1' AS traffic_state,
       TO_TIMESTAMP(a.`timeStamp`, 'yyyy-MM-dd HH:mm:ss:SSS') statistic_time,
       '10m',
       now(),
       t.trolley_equivalent,
       t.lane_flow_rate,
       t.vehicle_nums_ratio_mean,
       t.vehicle_length_ratio_mean,
       t.time_occupancy,
       t.static_queue_length_max,
       t.static_queue_length_min,
       t.dynamic_queue_length_max,
       t.dynamic_queue_length_min,
       t.light_green_start_queue_length,
       t.light_green_finish_queue_length,
       t.green_light_efficiency,
       t.lane_saturation_flow_rate,
       t.lane_capacity,
       t.lane_no_stop_rate,
       t.lane_three_stop_rate,
       TIMESTAMPADD(MINUTE,-5,TO_TIMESTAMP(DATE_FORMAT(a.`timeStamp`,'yyyy-MM-dd HH:mm:00'), 'yyyy-MM-dd HH:mm:ss')) start_window_time,
       TO_TIMESTAMP(DATE_FORMAT(a.`timeStamp`,'yyyy-MM-dd HH:mm:00'), 'yyyy-MM-dd HH:mm:ss') end_window_time,
       CAST(DATE_FORMAT(a.`timeStamp`,'yyyyMMdd') AS INT) dt,
       a.process_time,
       ifnull(t.traffic_flow_A,0) traffic_flow_A,
       ifnull(t.traffic_flow_B,0) traffic_flow_B,
       ifnull(t.traffic_flow_C,0) traffic_flow_C,
       ifnull(t.mean_len,0) mean_len,
       ifnull(t.time_headway_section_mean,0) time_headway_section_mean,
       ifnull(t.traffic_infomation,0) traffic_infomation,
       ifnull(e.free_speed,80) free_speed,
       d.is_signal,
       lineLengthUdf(c.wkt)/1000 as ridLength,
       a.ts_ltz
FROM source_gloabl_lane_cycle_data AS a
         CROSS JOIN  UNNEST(eventList) AS t(laneId,traffic_flow,trolley_equivalent,lane_flow_rate,mean_v,vehicle_nums_ratio_mean,vehicle_length_ratio_mean,time_occupancy,static_queue_length_max,static_queue_length_min,dynamic_queue_length_max,dynamic_queue_length_min,light_green_start_queue_length,light_green_finish_queue_length,green_light_efficiency,lane_saturation_flow_rate,lane_capacity,lane_saturation,lane_no_stop_rate,lane_one_stop_rate,lane_two_stop_rate,lane_three_stop_rate,mean_delay,mean_stops_number,traffic_flow_A,traffic_flow_B,traffic_flow_C,mean_len,time_headway_section_mean,traffic_infomation)
         LEFT JOIN t_base_lane_info FOR SYSTEM_TIME AS OF a.process_time AS b ON t.laneId=b.id
         LEFT JOIN t_base_rid_info FOR SYSTEM_TIME AS OF a.process_time AS c ON b.rid=c.id
         LEFT JOIN t_base_cross_info FOR SYSTEM_TIME AS OF a.process_time AS d ON c.end_cross_id=d.id
         LEFT JOIN t_cross_rid_freespeed  FOR SYSTEM_TIME AS OF a.process_time AS e ON e.rid=b.rid and e.rid=c.id
;

-- 车道级指标
insert into t_analysis_rid_lane_indicators(cross_id,cross_name,rid,rid_dir,segment_id,lane_id,lane_no,turn_dir_no,flow,saturation,delay_time,speed,queue_length,stop_num,once_stop_rate,second_stop_rate,traffic_state,statistic_time,granularity,ruksj,trolley_equivalent,lane_flow_rate,vehicle_nums_ratio_mean,vehicle_length_ratio_mean,time_occupancy,static_queue_length_max,static_queue_length_min,dynamic_queue_length_max,dynamic_queue_length_min,light_green_start_queue_length,light_green_finish_queue_length,green_light_efficiency,saturation_flow_rate,capacity,no_stop_rate,three_stop_rate,window_start_time,window_end_time,dt,traffic_flow_A,traffic_flow_B,traffic_flow_C,mean_len,time_headway_section_mean)
SELECT tb.cross_id,
       tb.cross_name,
       tb.rid,
       tb.in_dir,
       tb.segment_id,
       tb.laneId,
       tb.lane_no,
       tb.turn,
       SUM(tb.traffic_flow),
       SUM(tb.traffic_flow)/SUM(tb.lane_capacity) lane_saturation,
       AVG(tb.mean_delay),
       AVG(tb.mean_v),
       MAX(tb.queue_length),
       AVG(tb.mean_stops_number),
       AVG(tb.lane_one_stop_rate),
       AVG(tb.lane_two_stop_rate),
       (case
            when tb.is_signal=1 and AVG(tb.mean_delay) >=0 and AVG(tb.mean_delay)<55 then '1'
            when tb.is_signal=1 and AVG(tb.mean_delay)>=55 and AVG(tb.mean_delay)<100 then '2'
            when tb.is_signal=1 and AVG(tb.mean_delay)>=100 and AVG(tb.mean_delay)<145 then '3'
            when tb.is_signal=1 and AVG(tb.mean_delay)>=145  then '4'
            when tb.is_signal=0 and AVG(tb.mean_delay)>=0 and AVG(tb.mean_delay)<35 then '1'
            when tb.is_signal=0 and AVG(tb.mean_delay)>=35 and AVG(tb.mean_delay)<50 then '2'
            when tb.is_signal=0 and AVG(tb.mean_delay)>=50 and AVG(tb.mean_delay)<70 then '3'
            when tb.is_signal=0 and AVG(tb.mean_delay)>=70  then '4'
           end
           ) traffic_state,
       TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE)  statistic_time,
       '10m' granularity,
       now() ruksj,
       SUM(tb.trolley_equivalent),
       AVG(tb.lane_flow_rate),
       AVG(tb.vehicle_nums_ratio_mean),
       AVG(tb.vehicle_length_ratio_mean),
       AVG(tb.time_occupancy),
       MAX(tb.static_queue_length_max),
       MIN(tb.static_queue_length_min),
       MAX(tb.dynamic_queue_length_max),
       MIN(tb.dynamic_queue_length_min),
       AVG(tb.light_green_start_queue_length),
       AVG(tb.light_green_finish_queue_length),
       AVG(tb.green_light_efficiency),
       MAX(tb.lane_saturation_flow_rate),
       SUM(tb.lane_capacity),
       AVG(tb.lane_no_stop_rate),
       AVG(tb.lane_three_stop_rate),
       TUMBLE_START(tb.ts_ltz, INTERVAL '10' MINUTE)  start_window_time,
       TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE)  end_window_time,
       CAST(DATE_FORMAT(TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE),'yyyyMMdd') AS INT) AS dt,
       SUM(tb.traffic_flow_A),
       SUM(tb.traffic_flow_B),
       SUM(tb.traffic_flow_C),
       AVG(tb.mean_len),
       AVG(tb.time_headway_section_mean)
FROM v_kafka_cycle_data as tb
GROUP BY TUMBLE(tb.ts_ltz, INTERVAL '10' MINUTE),
         tb.cross_id,
         tb.is_signal,
         tb.cross_name,
         tb.rid,
         tb.in_dir,
         tb.segment_id,
         tb.laneId,
         tb.lane_no,
         tb.turn
;
-- ----------------------------------------------------------------------------------------------------------
create table t_analysis_rid_entrace_indicator
(
    cross_id VARCHAR,
    cross_name VARCHAR,
    rid VARCHAR,
    rid_dir INT,
    traffic_index DECIMAL(6,2),
    traffic_state VARCHAR,
    capacity INT,
    flow INT,
    saturation DECIMAL(6,2),
    queue_length INT,
    speed DECIMAL(6,2),
    delay_time DECIMAL(6,2),
    stop_num DECIMAL(6,2),
    statistic_time TIMESTAMP(3),
    granularity VARCHAR,
    ruksj TIMESTAMP(3),
    window_start_time TIMESTAMP(3),
    window_end_time TIMESTAMP(3),
    green_usage_rate DECIMAL(6,2),
    once_stop_rate DECIMAL(6,2),
    second_stop_rate DECIMAL(6,2),
    three_stop_rate DECIMAL(6,2),
    no_stop_rate DECIMAL(6,2),
    trolley_equivalent INT,
    rid_flow_rate DECIMAL(6,2),
    vehicle_nums_ratio_mean DECIMAL(6,2),
    vehicle_length_ratio_mean DECIMAL(6,2),
    time_occupancy DECIMAL(6,2),
    static_queue_length_max DECIMAL(6,2),
    static_queue_length_min DECIMAL(6,2),
    dynamic_queue_length_max DECIMAL(6,2),
    dynamic_queue_length_min DECIMAL(6,2),
    light_green_start_queue_length DECIMAL(6,2),
    light_green_finish_queue_length DECIMAL(6,2),
    saturation_flow_rate INT,
    traffic_flow_A INT,
    traffic_flow_B INT,
    traffic_flow_C INT,
    mean_len DECIMAL(6,2),
    time_headway_section_mean DECIMAL(6,2),
    PRIMARY KEY (rid,granularity,window_end_time) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_rid_entrace_indicator',
      'username' = 'root',
      'password' = 'Wanji300552'
      );
-- 进口道指标
insert into  t_analysis_rid_entrace_indicator(cross_id,cross_name,rid,rid_dir,traffic_index,capacity,flow,saturation,queue_length,speed,delay_time,stop_num,statistic_time,granularity,ruksj,window_start_time,window_end_time,
                                              green_usage_rate,once_stop_rate,second_stop_rate,three_stop_rate,no_stop_rate,trolley_equivalent,rid_flow_rate,vehicle_nums_ratio_mean,vehicle_length_ratio_mean,time_occupancy,
                                              static_queue_length_max,static_queue_length_min,dynamic_queue_length_max,dynamic_queue_length_min,light_green_start_queue_length,light_green_finish_queue_length,saturation_flow_rate,
                                              traffic_flow_A,traffic_flow_B,traffic_flow_C,mean_len,time_headway_section_mean,traffic_state
)
SELECT tb1.cross_id,tb1.cross_name,tb1.rid,tb1.in_dir,
       if(tb1.traffic_index>999,999,tb1.traffic_index),
       tb1.lane_capacity,
       tb1.traffic_flow,
       (tb1.traffic_flow/tb1.lane_capacity) saturation,
       tb1.queue_length,
       tb1.mean_v,
       tb1.mean_delay,
       tb1.mean_stops_number,
       tb1.statistic_time,
       tb1.granularity,
       tb1.ruksj,
       tb1.start_window_time,
       tb1.end_window_time,
       tb1.green_light_efficiency,
       tb1.lane_one_stop_rate,
       tb1.lane_two_stop_rate,
       tb1.lane_three_stop_rate,
       tb1.lane_no_stop_rate,
       tb1.trolley_equivalent,
       tb1.lane_flow_rate ,
       tb1.vehicle_nums_ratio_mean,
       tb1.vehicle_length_ratio_mean,
       tb1.time_occupancy,
       tb1.static_queue_length_max,
       tb1.static_queue_length_min,
       tb1.dynamic_queue_length_max,
       tb1.dynamic_queue_length_min,
       tb1.light_green_start_queue_length,
       tb1.light_green_finish_queue_length,
       tb1.lane_saturation_flow_rate,
       tb1.traffic_flow_A,
       tb1.traffic_flow_B,
       tb1.traffic_flow_C,
       tb1.mean_len,
       tb1.time_headway_section_mean,
       (case
            when tb1.is_signal=1 and tb1.mean_delay>=0 and tb1.mean_delay<55 then '1'
            when tb1.is_signal=1 and tb1.mean_delay>=55 and tb1.mean_delay<100 then '2'
            when tb1.is_signal=1 and tb1.mean_delay>=100 and tb1.mean_delay<145 then '3'
            when tb1.is_signal=1 and tb1.mean_delay>=145  then '4'
            when tb1.is_signal=0 and tb1.mean_delay>=0 and tb1.mean_delay<35 then '1'
            when tb1.is_signal=0 and tb1.mean_delay>=35 and tb1.mean_delay<50 then '2'
            when tb1.is_signal=0 and tb1.mean_delay>=50 and tb1.mean_delay<70 then '3'
            when tb1.is_signal=0 and tb1.mean_delay>=70  then '4'
           end
           ) traffic_state
FROM (
         SELECT tb.cross_id,tb.cross_name,tb.rid,tb.in_dir,
                (ifnull(tb.free_speed,80)/avg(tb.mean_v)) as traffic_index,
                SUM(tb.lane_capacity) lane_capacity,
                SUM(tb.traffic_flow) traffic_flow,
                AVG(tb.lane_saturation) lane_saturation,
                MAX(tb.queue_length) queue_length,
                AVG(tb.mean_v) mean_v,
                AVG(tb.mean_delay) mean_delay,
                AVG(tb.mean_stops_number) mean_stops_number,
                TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE)   statistic_time,
                '10m' granularity,
                now() ruksj,
                TUMBLE_START(tb.ts_ltz, INTERVAL '10' MINUTE)  start_window_time,
                TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE)  end_window_time,
                AVG( tb.green_light_efficiency ) green_light_efficiency,
                AVG( tb.lane_one_stop_rate ) lane_one_stop_rate,
                AVG( tb.lane_two_stop_rate ) lane_two_stop_rate,
                AVG( tb.lane_three_stop_rate ) lane_three_stop_rate,
                AVG( tb.lane_no_stop_rate ) lane_no_stop_rate,
                SUM( tb.trolley_equivalent ) trolley_equivalent,
                AVG( tb.lane_flow_rate)  lane_flow_rate ,
                AVG(tb.vehicle_nums_ratio_mean  ) vehicle_nums_ratio_mean,
                AVG(tb.vehicle_length_ratio_mean ) vehicle_length_ratio_mean,
                AVG(tb.time_occupancy ) time_occupancy,
                MAX(tb.static_queue_length_max) static_queue_length_max,
                MIN(tb.static_queue_length_min) static_queue_length_min,
                MAX(tb.dynamic_queue_length_max) dynamic_queue_length_max,
                MIN(tb.dynamic_queue_length_min) dynamic_queue_length_min,
                AVG(tb.light_green_start_queue_length ) light_green_start_queue_length,
                AVG(tb.light_green_finish_queue_length ) light_green_finish_queue_length,
                MAX(tb.lane_saturation_flow_rate) lane_saturation_flow_rate,
                SUM(traffic_flow_A) traffic_flow_A,
                SUM(traffic_flow_B) traffic_flow_B,
                SUM(traffic_flow_C) traffic_flow_C,
                AVG(mean_len) mean_len,
                AVG(time_headway_section_mean) time_headway_section_mean,
                tb.is_signal
         FROM v_kafka_cycle_data tb
         GROUP BY TUMBLE(tb.ts_ltz, INTERVAL '10' MINUTE),
                  tb.cross_id,tb.cross_name,tb.rid,tb.in_dir,tb.free_speed,tb.is_signal
     ) tb1
;

-- 转向流量计算
create table t_analysis_rid_turn_indicators
(
    cross_id  VARCHAR,
    f_rid  VARCHAR,
    f_rid_dir INT,
    turn_dir_no INT,
    flow INT,
    speed DECIMAL(6,2),
    queue_length INT,
    green_usage_rate DECIMAL(6,2),
    stop_num DECIMAL(6,2),
    once_stop_rate DECIMAL(6,2),
    second_stop_rate DECIMAL(6,2),
    traffic_state  VARCHAR,
    statistic_time TIMESTAMP(3),
    granularity  VARCHAR,
    ruksj TIMESTAMP(3),
    window_start_time TIMESTAMP(3),
    window_end_time TIMESTAMP(3),
    cross_name  VARCHAR,
    three_stop_rate DECIMAL(6,2),
    no_stop_rate DECIMAL(6,2),
    trolley_equivalent INT,
    turn_flow_rate   DECIMAL(6,2),
    vehicle_nums_ratio_mean  DECIMAL(6,2),
    vehicle_length_ratio_mean DECIMAL(6,2),
    time_occupancy DECIMAL(6,2),
    static_queue_length_max DECIMAL(6,2),
    static_queue_length_min DECIMAL(6,2),
    dynamic_queue_length_max DECIMAL(6,2),
    dynamic_queue_length_min DECIMAL(6,2),
    light_green_start_queue_length DECIMAL(6,2),
    light_green_finish_queue_length DECIMAL(6,2),
    saturation_flow_rate INT,
    capacity INT,
    traffic_flow_A INT,
    traffic_flow_B INT,
    traffic_flow_C INT,
    mean_len DECIMAL(6,2),
    time_headway_section_mean  DECIMAL(6,2),
    saturation  DECIMAL(6,2),
    delay_time DECIMAL(6,2),
    PRIMARY KEY (f_rid,turn_dir_no,granularity,window_end_time) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_rid_turn_indicators',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

create table t_config_lane_flow_coefficient
(
    id  VARCHAR,
    lane_function INT,
    lane_turn INT,
    flow_coefficient DECIMAL(6,2)
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_config_lane_flow_coefficient',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

INSERT INTO t_analysis_rid_turn_indicators (
    cross_id,
    cross_name,
    f_rid,
    f_rid_dir,
    turn_dir_no,
    flow,
    speed,
    queue_length,
    green_usage_rate,
    stop_num,
    once_stop_rate,
    second_stop_rate,
    three_stop_rate,
    no_stop_rate,
    traffic_state,
    statistic_time,
    granularity,
    ruksj,
    window_start_time,
    window_end_time ,
    trolley_equivalent ,
    turn_flow_rate  ,
    vehicle_nums_ratio_mean ,
    vehicle_length_ratio_mean,
    time_occupancy ,
    static_queue_length_max,
    static_queue_length_min,
    dynamic_queue_length_max,
    dynamic_queue_length_min,
    light_green_start_queue_length,
    light_green_finish_queue_length,
    saturation_flow_rate,
    capacity,
    traffic_flow_A ,
    traffic_flow_B ,
    traffic_flow_C ,
    mean_len ,
    time_headway_section_mean ,
    saturation,
    delay_time
)
SELECT
    tb1.cross_id,
    tb1.cross_name,
    tb1.rid,
    tb1.in_dir,
    tb1.lane_turn,
    tb1.traffic_flow,
    tb1.mean_v,
    tb1.queue_length,
    tb1.green_light_efficiency,
    tb1.mean_stops_number,
    tb1.lane_one_stop_rate,
    tb1.lane_two_stop_rate,
    tb1.lane_three_stop_rate,
    tb1.lane_no_stop_rate,
    -- IFNULL((select tg.traffic_state from t_config_congest_grade tg where tg.is_signal=tb1.is_signal and tb1.mean_delay>=tg.min_delay_time and tb1.mean_delay<tg.max_delay_time),'1') as traffic_state,
    (case
         when tb1.is_signal=1 and tb1.mean_delay>=0 and tb1.mean_delay<55 then '1'
         when tb1.is_signal=1 and tb1.mean_delay>=55 and tb1.mean_delay<100 then '2'
         when tb1.is_signal=1 and tb1.mean_delay>=100 and tb1.mean_delay<145 then '3'
         when tb1.is_signal=1 and tb1.mean_delay>=145  then '4'
         when tb1.is_signal=0 and tb1.mean_delay>=0 and tb1.mean_delay<35 then '1'
         when tb1.is_signal=0 and tb1.mean_delay>=35 and tb1.mean_delay<50 then '2'
         when tb1.is_signal=0 and tb1.mean_delay>=50 and tb1.mean_delay<70 then '3'
         when tb1.is_signal=0 and tb1.mean_delay>=70  then '4'
        end
        )  traffic_state,
    tb1.statistic_time,
    tb1.granularity,
    tb1.ruksj,
    tb1.start_window_time,
    tb1.end_window_time ,
    tb1.trolley_equivalent ,
    tb1.turn_flow_rate  ,
    tb1.vehicle_nums_ratio_mean,
    tb1.vehicle_length_ratio_mean,
    tb1.time_occupancy,
    tb1.static_queue_length_max,
    tb1.static_queue_length_min,
    tb1.dynamic_queue_length_max,
    tb1.dynamic_queue_length_min,
    tb1.light_green_start_queue_length,
    tb1.light_green_finish_queue_length,
    tb1.saturation_flow_rate,
    tb1.capacity,
    tb1.traffic_flow_A,
    tb1.traffic_flow_B,
    tb1.traffic_flow_C,
    tb1.mean_len,
    tb1.time_headway_section_mean,
    (tb1.traffic_flow/tb1.capacity) saturation,
    tb1.mean_delay
FROM
    (
        SELECT
            tb.cross_id,
            tb.cross_name,
            tb.rid,
            tb.in_dir,
            tb.lane_turn,
            CAST(ROUND(SUM( tb.traffic_flow * tb.flow_coefficient )) AS INT) traffic_flow,
            AVG( tb.mean_v * tb.flow_coefficient) mean_v,
            MAX( tb.queue_length ) queue_length,
            AVG( tb.green_light_efficiency ) green_light_efficiency,
            ROUND(AVG( tb.mean_stops_number * tb.flow_coefficient )) mean_stops_number,
            AVG( tb.lane_one_stop_rate ) lane_one_stop_rate,
            AVG( tb.lane_two_stop_rate ) lane_two_stop_rate,
            AVG( tb.lane_three_stop_rate ) lane_three_stop_rate,
            AVG( tb.lane_no_stop_rate ) lane_no_stop_rate,
            TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE) statistic_time,
            '10m' granularity,
            now() ruksj,
            TUMBLE_START(tb.ts_ltz, INTERVAL '10' MINUTE)  start_window_time,
            TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE)  end_window_time,
            CAST(ROUND(SUM( tb.trolley_equivalent * tb.flow_coefficient )) AS INT) trolley_equivalent ,
            AVG( tb.lane_flow_rate * tb.flow_coefficient ) turn_flow_rate  ,
            AVG(vehicle_nums_ratio_mean) vehicle_nums_ratio_mean,
            AVG(vehicle_length_ratio_mean) vehicle_length_ratio_mean,
            AVG(time_occupancy ) time_occupancy,
            MAX(static_queue_length_max) static_queue_length_max,
            MIN(static_queue_length_min) static_queue_length_min,
            MAX(dynamic_queue_length_max) dynamic_queue_length_max,
            MIN(dynamic_queue_length_min) dynamic_queue_length_min,
            AVG(light_green_start_queue_length ) light_green_start_queue_length,
            AVG(light_green_finish_queue_length ) light_green_finish_queue_length,
            CAST(ROUND(MAX( tb.lane_saturation_flow_rate * tb.flow_coefficient )) AS INT) saturation_flow_rate,
            CAST(ROUND(SUM( tb.lane_capacity * tb.flow_coefficient )) AS INT) capacity,
            CAST(ROUND(SUM(traffic_flow_A * tb.flow_coefficient),0) AS INT) traffic_flow_A,
            CAST(ROUND(SUM(traffic_flow_B * tb.flow_coefficient),0) AS INT) traffic_flow_B,
            CAST(ROUND(SUM(traffic_flow_C * tb.flow_coefficient),0) AS INT) traffic_flow_C,
            AVG(mean_len) mean_len,
            AVG(time_headway_section_mean) time_headway_section_mean,
            AVG(tb.lane_saturation * tb.flow_coefficient ) saturation,
            AVG(mean_delay) mean_delay,
            tb.is_signal
        FROM (
                 select * from v_kafka_cycle_data t1
                                   LEFT JOIN t_config_lane_flow_coefficient FOR SYSTEM_TIME AS OF t1.process_time AS t2 ON t2.lane_function = t1.turn
             ) tb
        GROUP BY TUMBLE(tb.ts_ltz, INTERVAL '10' MINUTE),
                 tb.cross_id,
                 tb.cross_name,
                 tb.rid,
                 tb.in_dir,
                 tb.lane_turn,
                 tb.is_signal,
                 tb.free_speed
    ) tb1
;


-- 路口指标---------------------------------------------
create table t_analysis_cross_indicators
(
    cross_id	varchar,
    cross_name	varchar,
    speed	decimal(6,2),
    saturation	decimal(6,2),
    stop_num	decimal(6,2),
    max_queue_length	int,
    ruksj	TIMESTAMP(3),
    granularity	varchar,
    avg_delay	 decimal(6,2),
    capacity	int,
    flow	int,
    window_start_time	TIMESTAMP(3),
    window_end_time	TIMESTAMP(3),
    congest_index decimal(6,2),
    no_stop_rate decimal(6,2),
    once_stop_rate decimal(6,2),
    second_stop_rate decimal(6,2),
    three_stop_rate decimal(6,2),
    green_start_max_queue INT,
    green_use_rate	decimal(6,2),
    hour_flow int,
    efficiency_index 	decimal(6,2),
    evaluation_grade varchar,
    service_level varchar,
    PRIMARY KEY (cross_id,granularity,window_end_time) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_cross_indicators',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

-- 计算路口通过时长占比
CREATE VIEW v_pass_time_rate AS
SELECT tc.cross_id, SUM(tc.free_travel_time) free_travel_time,SUM(tc.travel_time) as travel_time, tc.window_start,tc.window_end
FROM (
         SELECT  tb.cross_id,
                 tb.rid,
                 (tb.ridLength / tb.free_speed)  free_travel_time,
                 (tb.ridLength / if(avg(tb.mean_v)=0,1,avg(tb.mean_v))) travel_time,
                 window_start, window_end
         FROM TABLE(TUMBLE(TABLE v_kafka_cycle_data,DESCRIPTOR(ts_ltz),INTERVAL '10' MINUTE)) tb
         GROUP BY tb.cross_id, tb.cross_name, tb.rid, tb.ridLength, tb.free_speed, window_start, window_end
     ) tc
GROUP BY tc.cross_id, tc.window_start, tc.window_end
;


insert into t_analysis_cross_indicators(cross_id,cross_name,speed,saturation,stop_num,max_queue_length,ruksj,granularity,avg_delay,capacity,flow,window_start_time,window_end_time,congest_index,no_stop_rate,once_stop_rate,second_stop_rate,three_stop_rate,green_start_max_queue,green_use_rate,hour_flow,efficiency_index,evaluation_grade,service_level)
SELECT t.*,
       -- 效率等级
       (CASE WHEN t.efficiency_index>=7 THEN '1' WHEN t.efficiency_index>=3 THEN '2' ELSE '3' END) evaluation_grade,
       -- 服务水平
       (
           CASE WHEN t.saturation<0.4 THEN 'A'
                WHEN t.saturation>=0.4 and t.saturation<0.6 THEN 'B'
                WHEN t.saturation>=0.6 and t.saturation<0.75 THEN 'C'
                WHEN t.saturation>=0.75 and t.saturation<0.9 THEN 'D'
                WHEN t.saturation>=0.9 and t.saturation<1 THEN 'E'
                ELSE 'F'
               END
           ) service_level
FROM
    (
        SELECT  t1.cross_id,t1.cross_name,t1.speed,t1.saturation,t1.stop_num,t1.max_queue_length,LOCALTIMESTAMP,t1.granularity,t1.avg_delay,t1.capacity,t1.flow,t1.start_window_time,t1.end_window_time,t1.congest_index,t1.no_stop_rate,t1.once_stop_rate,t1.second_stop_rate,t1.three_stop_rate,t1.green_start_max_queue,t1.green_use_rate,t1.hour_flow,
                (
                            (t1.flow/t1.capacity)  * 10 * 0.2 +
                            (t1.avg_lane_saturation)  * 10 * 0.2 +
                            (t1.no_stop_rate +  t1.once_stop_rate) * 10 * 0.3 +
                            (t2.free_travel_time / t2.travel_time) * 10 * 0.3
                    )/4 AS efficiency_index  -- 效率指数
        FROM (
                 SELECT tb.cross_id,
                        tb.cross_name,
                        AVG(tb.mean_v) as speed,
                        SUM(tb.traffic_flow)/ SUM(tb.lane_capacity) saturation,
                        AVG(tb.mean_stops_number) stop_num,
                        CAST(ROUND(MAX(tb.queue_length),0) AS INT) max_queue_length,
                        '10m' granularity,
                        AVG(tb.mean_delay) avg_delay,
                        SUM(tb.lane_capacity ) capacity,
                        SUM(tb.traffic_flow) flow,
                        TUMBLE_START(tb.ts_ltz, INTERVAL '10' MINUTE)  start_window_time,
                        TUMBLE_END(tb.ts_ltz, INTERVAL '10' MINUTE)  end_window_time,
                        if( (MAX(ifnull(tb.free_speed,80))/avg(tb.mean_v)) >999, 999, (MAX(ifnull(tb.free_speed,80))/avg(tb.mean_v)) ) as congest_index,
                        AVG( tb.lane_no_stop_rate ) no_stop_rate,
                        AVG( tb.lane_one_stop_rate ) once_stop_rate,
                        AVG( tb.lane_two_stop_rate ) second_stop_rate,
                        AVG( tb.lane_three_stop_rate ) three_stop_rate,
                        CAST(MAX(tb.light_green_start_queue_length) AS INT) green_start_max_queue,
                        AVG( tb.green_light_efficiency)  green_use_rate,
                        CAST(AVG(tb.lane_flow_rate) AS INT) hour_flow,
                        AVG(tb.lane_saturation) avg_lane_saturation
                 FROM v_kafka_cycle_data tb
                 group by  TUMBLE(tb.ts_ltz, INTERVAL '10' MINUTE),tb.cross_id,tb.cross_name
             ) t1
                 FULL JOIN v_pass_time_rate t2 on t1.cross_id=t2.cross_id and t1.start_window_time=t2.window_start and t1.end_window_time=t2.window_end
    ) t
;