-- -- 开启 mini-batch 指定是否启用小批量优化 （相关配置说明 https://ci.apache.org/projects/flink/flink-docs-release-1.12/zh/dev/table/config.html）
-- SET table.exec.mini-batch.enabled=true;
-- -- mini-batch的时间间隔，即作业需要额外忍受的延迟
-- SET table.exec.mini-batch.allow-latency=60s;
-- -- 一个 mini-batch 中允许最多缓存的数据
-- SET table.exec.mini-batch.size=5;
set table.exec.sink.not-null-enforcer=drop;
-- set table.exec.resource.default-parallelism=2;
-- 数据源kafka数据
CREATE TABLE kafka_vehicle_track_source (
                                            `timeStamp` VARCHAR,
                                            globalTimeStamp BIGINT,
                                            participantNum INT,
                                            orgCode VARCHAR,
                                            e1FrameParticipant ARRAY<
                                                ROW<
                                                id INT,
                                            licenseColor INT,
                                            picLicense VARCHAR,
                                            originalColor INT,
                                            originalType INT,
                                            longitude DOUBLE,
                                            latitude DOUBLE,
                                            speed DOUBLE,
                                            altitude DOUBLE,
                                            courseAngle DOUBLE,
                                            baseLocation INT,
                                            waitingTime INT,
                                            stopNum INT,
                                            roadnet MAP<String,String>
                                                >
                                                >,
                                            ts_ltz AS TO_TIMESTAMP_LTZ(globalTimeStamp, 3),
                                            WATERMARK FOR ts_ltz AS ts_ltz - INTERVAL '5' SECOND,
                                            process_time AS PROCTIME ()
) WITH (
      'connector' = 'kafka', -- 使用 kafka connector
      'topic' = 'JNMatchResultMiniData',  -- kafka topic
      'properties.bootstrap.servers' = '192.168.2.103:9092',  -- broker连接信息
      'properties.group.id' = 'frame-data-source_area_idx_calc', -- 消费kafka的group_id
      'properties.enable.auto.commit' = 'true',
      'properties.auto.commit.interval.ms' = '1000',
      'scan.startup.mode' = 'group-offsets',
      -- 'scan.startup.mode' = 'latest-offset',
      --  'scan.startup.mode' = 'earliest-offset',
      -- 'scan.startup.timestamp-millis' = '1689436800000',
      -- 'scan.startup.mode' = 'timestamp',  -- 读取数据的位置
      'format' = 'json',  -- 数据源格式为 json
      'json.fail-on-missing-field' = 'false', -- 字段丢失任务不失败
      'json.ignore-parse-errors' = 'true'  -- 解析失败跳过
      ) ;

CREATE TABLE t_config_participant_type (
                                           category INT,
                                           original_type INT
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_config_participant_type',
      'username' = 'root',
      'password' = 'Wanji300552'
      );


CREATE TABLE t_analysis_ped_nonmotor(
                                        ped_number INT,
                                        nonmotor_number INT,
                                        motor_speed INT,
                                        ruksj TIMESTAMP(3),
                                        window_start_time TIMESTAMP(3),
                                        window_end_time TIMESTAMP(3),
                                        granularity VARCHAR,
                                        PRIMARY KEY (granularity,window_end_time) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_ped_nonmotor',
      'username' = 'root',
      'password' = 'Wanji300552',
      'sink.buffer-flush.max-rows' = '0',
      'sink.buffer-flush.interval' = '1000'
      );

-- 路段rid表
create table t_base_rid_info (
                                 id VARCHAR ,-- 路段ID
                                 in_dir INT, -- 路段方向
                                 end_cross_id VARCHAR-- 终点路口
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8',
      'table-name' = 't_base_rid_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

CREATE TABLE t_cross_rid_freespeed_q (
                                         cross_id VARCHAR,
                                         rid VARCHAR,
                                         free_speed DECIMAL(6,2),
                                         window_start_time TIMESTAMP(3),
                                         window_end_time TIMESTAMP(3)
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_cross_rid_freespeed',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

create table t_base_lane_info (
                                  cross_id VARCHAR, -- 路口ID
                                  id VARCHAR ,-- 车道ID
                                  rid VARCHAR, -- 路段ID
                                  dir INT,-- 路段方向
                                  sort INT, -- 车道编号
                                  turn INT, -- 车道功能
                                  segment_id VARCHAR, -- 渠化ID
                                  type INT , -- 1：路段车道 2：进口车道 3：出口车道
                                  category INT -- 车道类别：1机动车；2非机动车；3公交专用；4可变；5潮汐
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8',
      'table-name' = 't_base_lane_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

CREATE TABLE t_analysis_ped_nonmotor_speed(
                                              rid VARCHAR,
                                              number BIGINT,
                                              speed DECIMAL(6,2),
                                              category INT,
                                              granularity VARCHAR,
                                              window_start_time TIMESTAMP(3),
                                              window_end_time TIMESTAMP(3),
                                              create_date TIMESTAMP(3),
                                              PRIMARY KEY (rid,category,granularity,window_start_time) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_ped_nonmotor_speed',
      'username' = 'root',
      'password' = 'Wanji300552',
      'sink.buffer-flush.max-rows' = '0',
      'sink.buffer-flush.interval' = '1000'
      );

-- 自由流速度计算
CREATE TABLE t_cross_rid_freespeed (
                                       cross_id VARCHAR,
                                       rid VARCHAR,
                                       free_speed DECIMAL(6,2),
                                       window_start_time TIMESTAMP(3),
                                       window_end_time TIMESTAMP(3),
                                       create_date TIMESTAMP(3),
                                       PRIMARY KEY (cross_id,rid) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_cross_rid_freespeed',
      'username' = 'root',
      'password' = 'Wanji300552',
      'sink.buffer-flush.max-rows' = '0',
      'sink.buffer-flush.interval' = '1000'
      );



CREATE TABLE t_analysis_area_index (
                                       traffic_index  DECIMAL(6,2),
                                       traffic_state VARCHAR,
                                       speed DECIMAL(6,2),
                                       in_way_vehicle_num BIGINT,
                                       local_vehicle_num BIGINT,
                                       out_vehicle_num BIGINT,
                                       window_start_time TIMESTAMP(3),
                                       window_end_time TIMESTAMP(3),
                                       statistic_time TIMESTAMP(3),
                                       granularity VARCHAR,
                                       travel_time DECIMAL(6,2),
                                       congest_length  DECIMAL(6,2),
                                       PRIMARY KEY(granularity,window_start_time,window_end_time) NOT ENFORCED
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://172.17.0.1:3306/holo_roadnet?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_analysis_area_index',
      'username' = 'root',
      'password' = 'Wanji300552',
      'sink.buffer-flush.max-rows' = '0',
      'sink.buffer-flush.interval' = '1000'
      -- ,'sink.parallelism' = '2'
      );

CREATE VIEW v_realtime_track AS
SELECT c.cross_id,c.rid,e.free_speed,a.globalTimeStamp,a.ts_ltz,b.category,t.*  ,c.turn,c.category as lane_category,a.process_time
FROM kafka_vehicle_track_source a  CROSS JOIN UNNEST(e1FrameParticipant) AS t(id,licenseColor,picLicense,originalColor,originalType,longitude,latitude,speed,altitude,courseAngle,baseLocation,waitingTime,stopNum,roadnet)
                                   JOIN t_config_participant_type FOR SYSTEM_TIME AS OF a.process_time AS b ON t.originalType=b.original_type
                                   LEFT JOIN t_base_lane_info FOR SYSTEM_TIME AS OF a.process_time AS c ON t.roadnet['laneId']=c.id
    -- LEFT JOIN t_base_rid_info FOR SYSTEM_TIME AS OF a.process_time AS d ON c.rid=d.id
                                   LEFT JOIN t_cross_rid_freespeed_q  FOR SYSTEM_TIME AS OF a.process_time AS e  ON c.rid=e.rid
;

-- 区域相关指标计算
INSERT INTO t_analysis_area_index(traffic_index,traffic_state,speed,in_way_vehicle_num,local_vehicle_num,out_vehicle_num,window_start_time,window_end_time,statistic_time,granularity,travel_time,congest_length)
SELECT (ifnull(avg(t.free_speed),80)/avg(t.speed)) traffic_index,
       '1' as traffic_state,
       avg(IF(t.speed>1,t.speed ,cast(NULL as DOUBLE))) speed,
       count(t.id) in_way_vehicle_num,
       count(IF( t.localOutType=1 , t.localOutType ,cast(NULL as INTEGER))) local_vehicle_num,
       (count(t.id)-count(IF( t.localOutType=1 , t.localOutType ,cast(NULL as INTEGER)))) out_vehicle_num,
       window_start, window_end,
       -- TUMBLE_START(t.window_time, INTERVAL '10' MINUTE) window_start_time,
       --   TUMBLE_END(t.window_time, INTERVAL '10' MINUTE) window_end_time,
       LOCALTIMESTAMP,
       '10m',
       max(t.travelTime) travelTime,
       2330 as congest_length
FROM
    (  -- 对10分钟内每辆车计算
        SELECT
            window_start, window_end,window_time,
            id, -- id标识
            category, -- 目标分类 1机动 2非机动 3行人
            picLicense, -- 车牌号
            free_speed, -- 自由流速度
            avg(IF(speed>1,speed ,cast(NULL as DOUBLE))) speed, 				-- 同一个目标窗口内平均速度
            TIMESTAMPDIFF(
                SECOND,
                    to_timestamp(FROM_UNIXTIME(min(globalTimeStamp)/1000)),
                    to_timestamp(FROM_UNIXTIME(max(globalTimeStamp)/1000))
                ) travelTime, 							-- 目标在窗口内行程时间：秒
            (CASE WHEN LEFT(picLicense,1)='鲁' THEN 1 ELSE 0 END) localOutType 		-- 1本地车 2外地车
            -- FROM v_realtime_track
        FROM TABLE(
                TUMBLE(TABLE v_realtime_track, DESCRIPTOR(ts_ltz), INTERVAL '10' MINUTE)
            )
        WHERE category=1
        GROUP BY window_start, window_end,window_time, category,id,picLicense,free_speed
    ) t
GROUP BY  window_start, window_end
;


INSERT INTO t_analysis_ped_nonmotor_speed(rid,number,speed,category,granularity,window_start_time,window_end_time,create_date)
SELECT t.rid,
       count(distinct id) AS number,
       avg(speed) as speed,
       t.category,
       '15m',
       window_start,
       window_end,
       now()
FROM TABLE(TUMBLE(TABLE v_realtime_track,DESCRIPTOR(ts_ltz),INTERVAL '15' MINUTE))  t
GROUP BY window_start,window_end,t.rid,t.category
;

INSERT INTO t_cross_rid_freespeed(cross_id,rid,free_speed,window_start_time,window_end_time,create_date)
SELECT t.cross_id,t.rid,MAX(t.speed) as free_speed,TUMBLE_START(t.ts_ltz, INTERVAL '1' DAY),TUMBLE_END(t.ts_ltz, INTERVAL '1' DAY),now()
FROM v_realtime_track t
WHERE t.category=1
GROUP BY TUMBLE(t.ts_ltz, INTERVAL '1' DAY),t.rid ,t.cross_id
;

create table t_config_lane_flow_coefficient
(
    id  VARCHAR,
    lane_function INT,
    lane_turn INT,
    flow_coefficient DECIMAL(6,2)
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_cs?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
      'table-name' = 't_config_lane_flow_coefficient',
      'username' = 'root',
      'password' = 'Wanji300552'
      );

CREATE TABLE t_analysis_cross_non_motor (
                                            id STRING,
                                            cross_id STRING,
                                            turn_no INT,
                                            type INT,
                                            number INT,
                                            granularity STRING,
                                            window_start_time TIMESTAMP(3),
                                            window_end_time TIMESTAMP(3),
                                            create_date TIMESTAMP(3),
                                            PRIMARY KEY(cross_id,turn_no,type,granularity,window_start_time,window_end_time) NOT ENFORCED
)  WITH (
       'connector' = 'jdbc',
       'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
       'table-name' = 't_analysis_cross_non_motor',
       'username' = 'root',
       'password' = 'Wanji300552'
       );
-- 关联到转向比率分配表
CREATE VIEW v_realtime_track_turn AS
select * from v_realtime_track  t1
                  LEFT JOIN t_config_lane_flow_coefficient FOR SYSTEM_TIME AS OF t1.process_time AS t2 ON t2.lane_function  = t1.turn
;
-- 机动车道/非机动车道 上的非机动车数量（15分钟粒度）
INSERT INTO t_analysis_cross_non_motor(id,cross_id,turn_no,type,number,granularity,window_start_time,window_end_time,create_date)
SELECT
    uuid(),tb.cross_id,tb.lane_turn,tb.lane_category,SUM(tb.num),'15m',tb.window_start,tb.window_end,LOCALTIMESTAMP
FROM (
         SELECT
             t.cross_id,t.lane_turn,t.lane_category,CAST(count(DISTINCT t.id) * t.flow_coefficient AS INT) num,window_start,window_end
         FROM TABLE(TUMBLE(TABLE v_realtime_track_turn,DESCRIPTOR(ts_ltz),INTERVAL '15' MINUTE))  t
         WHERE t.category=2 and t.lane_category in (1,2)
         GROUP BY t.cross_id,t.rid,t.lane_turn,t.lane_category,window_start,window_end,t.flow_coefficient
     ) tb
GROUP BY  tb.cross_id,tb.lane_turn,tb.lane_category,tb.window_start,tb.window_end
;

-- 机动车道/非机动车道 上的非机动车数量（30分钟粒度）
INSERT INTO t_analysis_cross_non_motor(id,cross_id,turn_no,type,number,granularity,window_start_time,window_end_time,create_date)
SELECT
    uuid(),tb.cross_id,tb.lane_turn,tb.lane_category,SUM(tb.num),'15m',tb.window_start,tb.window_end,LOCALTIMESTAMP
FROM (
         SELECT
             t.cross_id,t.lane_turn,t.lane_category,CAST(count(DISTINCT t.id) * t.flow_coefficient AS INT) num,window_start,window_end
         FROM TABLE(TUMBLE(TABLE v_realtime_track_turn,DESCRIPTOR(ts_ltz),INTERVAL '30' MINUTE))  t
         WHERE t.category=2 and t.lane_category in (1,2)
         GROUP BY t.cross_id,t.rid,t.lane_turn,t.lane_category,window_start,window_end,t.flow_coefficient
     ) tb
GROUP BY  tb.cross_id,tb.lane_turn,tb.lane_category,tb.window_start,tb.window_end
;

-- 机动车道/非机动车道 上的非机动车数量（1小时粒度）
INSERT INTO t_analysis_cross_non_motor(id,cross_id,turn_no,type,number,granularity,window_start_time,window_end_time,create_date)
SELECT
    uuid(),tb.cross_id,tb.lane_turn,tb.lane_category,SUM(tb.num),'15m',tb.window_start,tb.window_end,LOCALTIMESTAMP
FROM (
         SELECT
             t.cross_id,t.lane_turn,t.lane_category,CAST(count(DISTINCT t.id) * t.flow_coefficient AS INT) num,window_start,window_end
         FROM TABLE(TUMBLE(TABLE v_realtime_track_turn,DESCRIPTOR(ts_ltz),INTERVAL '1' HOUR))  t
         WHERE t.category=2 and t.lane_category in (1,2)
         GROUP BY t.cross_id,t.rid,t.lane_turn,t.lane_category,window_start,window_end,t.flow_coefficient
     ) tb
GROUP BY  tb.cross_id,tb.lane_turn,tb.lane_category,tb.window_start,tb.window_end
;

