SET table.exec.mini-batch.enabled = true;
SET table.exec.mini-batch.allow-latency = 5s;
SET table.exec.mini-batch.size = 5000;
SET table.local-time-zone=Asia/Shanghai;

-- 数据源kafka数据
CREATE TABLE kafka_vehicle_snapshot_source (
                                               `timeStamp` VARCHAR,
                                               orgCode VARCHAR,
                                               laneNum INT,
                                               eventList ARRAY<
                                                   ROW<
                                                   laneId VARCHAR,
                                               static_queue_length DOUBLE,
                                               dynamic_queue_length  DOUBLE,
                                               mean_space_headway DOUBLE,
                                               car_nums  INT,
                                               vehicle_nums_ratio  DOUBLE,
                                               vehicle_length_ratio  DOUBLE,
                                               team_head_distance   DOUBLE,
                                               team_tail_distance  DOUBLE,
                                               std_space_headway  DOUBLE,
                                               queue_nums INT,
                                               mean_v  DOUBLE,
                                               head_car_position  DOUBLE,
                                               start_v  DOUBLE,
                                               tail_car_position  DOUBLE,
                                               end_v  DOUBLE
                                                   >
                                                   >,
                                               process_time AS PROCTIME ()
) WITH (
      'connector' = 'kafka', -- 使用 kafka connector
      'topic-pattern' = 'JN(.*)LaneSnapshotData',  -- kafka topic
      'properties.bootstrap.servers' = '10.102.1.182:9092',  -- broker连接信息
      'properties.group.id' = 'global_lane_snapshot_data', -- 消费kafka的group_id
      'properties.enable.auto.commit' = 'true',
      'properties.auto.commit.interval.ms' = '1000',
      -- 'scan.startup.mode' = 'group-offsets',
      -- 'scan.startup.mode' = 'latest-offset',  -- 读取数据的位置
      'scan.startup.mode' = 'earliest-offset',
      -- 'scan.startup.mode' = 'group-offsets',
      -- 'scan.startup.mode' = 'timestamp',  -- 读取数据的位置
      'scan.startup.timestamp-millis' = '1689120000000', -- 设定消费的时间戳起点
      'format' = 'json',  -- 数据源格式为 json
      'json.fail-on-missing-field' = 'false', -- 字段丢失任务不失败
      'json.ignore-parse-errors' = 'true'  -- 解析失败跳过
      ) ;
-- mysql sink 表
CREATE TABLE t_global_lane_snapshot_data  (
                                              cross_id VARCHAR,
                                              cross_name VARCHAR,
                                              rid VARCHAR,
                                              rid_dir INT,
                                              segment_id  VARCHAR,
                                              lane_id VARCHAR,
                                              lane_no INT,
                                              turn_dir_no INT,
                                              static_queue_length DECIMAL(6,2),
                                              dynamic_queue_length DECIMAL(6,2),
                                              mean_space_headway DECIMAL(6,2),
                                              car_nums INT,
                                              vehicle_nums_ratio DECIMAL(6,2),
                                              vehicle_length_ratio DECIMAL(6,2),
                                              frame_time VARCHAR,
                                              ruksj TIMESTAMP(3),
                                              dt INT,
                                              team_head_distance   DECIMAL(6,2),
                                              team_tail_distance  DECIMAL(6,2),
                                              std_space_headway  DECIMAL(6,2),
                                              queue_nums INT,
                                              mean_v  DECIMAL(6,2),
                                              head_car_position DECIMAL(6,2),
                                              start_v  DECIMAL(6,2),
                                              tail_car_position  DECIMAL(6,2),
                                              end_v  DECIMAL(6,2),
                                              PRIMARY KEY (lane_id, frame_time,dt) NOT ENFORCED
)  WITH (
       'connector' = 'jdbc',
       'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?rewriteBatchedStatements=true&characterEncoding=UTF-8&serverTimezone=Asia/Shanghai',
       'table-name' = 't_global_lane_snapshot_data',
       'username' = 'root',
       'password' = 'Wanji300552',
       'sink.parallelism' = '2',
       'sink.buffer-flush.max-rows' = '0',
       'sink.buffer-flush.interval' = '1500'
       );
create table t_base_lane_info (
                                  cross_id VARCHAR, -- 路口ID
                                  id VARCHAR ,-- 车道ID
                                  rid VARCHAR, -- 路段ID
                                  dir INT,-- 路段方向
                                  sort INT, -- 车道编号
                                  turn INT, -- 车道功能
                                  segment_id VARCHAR , -- 渠化ID
                                  type INT -- 1：路段车道 2：进口车道 3：出口车道
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8',
      'table-name' = 't_base_lane_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );
-- 路段rid表
create table t_base_rid_info (
                                 id VARCHAR ,-- 路段ID
                                 in_dir INT, -- 路段方向
                                 end_cross_id VARCHAR-- 终点路口
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8',
      'table-name' = 't_base_rid_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );
-- 路口表
create table t_base_cross_info (
                                   id VARCHAR ,-- 路口ID
                                   `name` VARCHAR -- 路口名称
) WITH (
      'connector' = 'jdbc',
      'url' = 'jdbc:mysql://10.102.1.182:3306/holo_roadnet_jinan?characterEncoding=UTF-8',
      'table-name' = 't_base_cross_info',
      'username' = 'root',
      'password' = 'Wanji300552'
      );
-- 同步到mysql
INSERT INTO t_global_lane_snapshot_data(frame_time,cross_id,cross_name,rid,rid_dir,segment_id,lane_id,lane_no,turn_dir_no,static_queue_length,dynamic_queue_length,mean_space_headway,car_nums,vehicle_nums_ratio,vehicle_length_ratio,ruksj,dt,team_head_distance,team_tail_distance,std_space_headway,queue_nums,mean_v,head_car_position,start_v,tail_car_position,end_v)
SELECT a.`timeStamp`,b.cross_id,d.`name`,b.rid,c.in_dir,b.segment_id,t.laneId,b.sort,b.turn,t.static_queue_length,t.dynamic_queue_length,t.mean_space_headway,t.car_nums,t.vehicle_nums_ratio,t.vehicle_length_ratio,now(),CAST(DATE_FORMAT(a.`timeStamp`,'yyyyMMddHH') AS INT),t.team_head_distance,t.team_tail_distance,t.std_space_headway,t.queue_nums,t.mean_v,t.head_car_position,t.start_v,t.tail_car_position,t.end_v
FROM kafka_vehicle_snapshot_source AS a
         CROSS JOIN  UNNEST(eventList) AS t(laneId,static_queue_length,dynamic_queue_length,mean_space_headway,car_nums,vehicle_nums_ratio,vehicle_length_ratio,team_head_distance,team_tail_distance,std_space_headway,queue_nums,mean_v,head_car_position,start_v,tail_car_position,end_v)
         LEFT JOIN t_base_lane_info FOR SYSTEM_TIME AS OF a.process_time AS b ON t.laneId=b.id and b.type=2
         LEFT JOIN t_base_rid_info FOR SYSTEM_TIME AS OF a.process_time AS c ON b.rid=c.id
         LEFT JOIN t_base_cross_info FOR SYSTEM_TIME AS OF a.process_time AS d ON c.end_cross_id=d.id
;
